#Licensed Materials - Property of IBM
#5724-L01, 5724-I82, 5655-N53, 5655-R15
#Copyright IBM Corp. 2006, 2008 
#All Rights Reserved.
#US Government Users Restricted Rights - Use, duplication or
#disclosure restricted by GSA ADP Schedule Contract with
#IBM Corp.

from java.lang import System
from java.io import File
from com.ibm.ws.install.ifactory.adapters import WBIContributionAdapter
import thread
import time

##########################
#  Class wbi_cadapter
##########################
class wbi_cadapter:
    def __init__(self):
        self._adapter = WBIContributionAdapter()
        self._killThread = 1   #Set 1 to start threading loop, set 0 to kill
        self._progress = 0     #Keeps track of progress returning from adapter class
        invocationResult.setInvocationExitCode(invocationId, invocationResult.UNKNOWN)   #Default to UNKNOWN unless it's set to another result after the installer ran

###############################################################################
#  wbi_cadapter - run method
#  Gets options, executable's path and java path for the contribution.
###############################################################################
    def run( self ):
        methodName="run"
        launcherName="setup.jar"
        separator = System.getProperty("file.separator")
        try:
            BDoptions = invocationData.getBDOptionsForCurrentPlatform (invocationId)
#           A java null is turned into a Python None value, throw exception if options is not resolved and null is received
            if BDoptions is None:
                raise exception
            if BDoptions != "":
                self._adapter.setBDOptions(BDoptions)
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.BDOptions", BDoptions))

            CMDoptions = invocationData.getCMDOptionsForCurrentPlatform (invocationId)
#           A java null is turned into a Python None value, throw exception if options is not resolved and null is received
            if CMDoptions is None:
                raise exception
            if CMDoptions != "":
                self._adapter.setCmdOptions(CMDoptions)
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.CMDOptions", CMDoptions))

#           Use install.exe for Windows Vista because elevated permission is required.
            if (System.getProperty ("os.name") == "Windows Vista"):
                launcherName="install.exe"

            cipSetupjar = invocationData.getContributionAbsolutePathForCurrentPlatform(invocationId) + separator + "wbiifpackage" + separator + "WBI" + separator + launcherName
            f_Setupjar = File(cipSetupjar)

            if (f_Setupjar.exists() and f_Setupjar.isFile()):
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.ExecutablePath", cipSetupjar))
                self._adapter.setExecutable(cipSetupjar)
            else:
                executable_path = invocationData.getContributionAbsolutePathForCurrentPlatform(invocationId) + separator + "WBI" + separator + launcherName
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.ExecutablePath", executable_path))
                self._adapter.setExecutable(executable_path)

            if (System.getProperty ("os.name") != "Windows Vista" and System.getProperty ("os.name") != "OS/400"):
                java_path = invocationData.getContributionAbsolutePathForCurrentPlatform(invocationId) + separator + "JDK" + separator + "jre.pak" + separator + "repository" + separator + "package.java.jre" + separator + "java" + separator + "jre" + separator + "bin" + separator + "java"
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.JavaPath", java_path))            
                self._adapter.setJavaExePath(java_path)

            if (System.getProperty ("os.name") == "OS/400"):
                java_path = "java"
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.JavaPath", java_path))
                self._adapter.setJavaExePath(java_path)
                class_path = invocationData.getContributionAbsolutePathForCurrentPlatform(invocationId) + separator + "WBI" + separator + "lib" + separator + "jt400.jar";
                self._adapter.setClasspath(class_path)

            if ( invocationData.canUserModifyThisOptionForCurrentPlatform(invocationId, "installLocation") ):
                self._adapter.setInstallLocationModifiableOn()
            else:
                self._adapter.setInstallLocationModifiableOff()
#           self._adapter.setAggregatedInstallableContentForInstall( invocationData.getAggregatedInstallableContentForThisInvocation (invocationId), invocationId )

            self._adapter.setInvocationId (invocationId)
            self._adapter.setIIPInvocationData(invocationData)
            self._adapter.execute()
#           Outputting invalid options passed in:
            strInvalidOptions = self._adapter.getInvalidOptionsTokenizedString()
            if (strInvalidOptions is not None and strInvalidOptions != ""):
                jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.InvalidOptions", strInvalidOptions))

            jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.EndExecution"))
        except:
            jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.ExecutionException"))
            self._killThread = 0
            
###############################################################################
#  wbi_cadapter - setResults method
#  Sets results received from adapter back to IIP installer.
###############################################################################
    def setResults( self ):
        methodName="setResults"
        jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.StartSetResults"))
        try:
            invocationResult.setInvocationMessage(invocationId, self._adapter.getInstallResultMessage())
            if (self._adapter.getInstallExitCode() == 0 or self._adapter.getInstallExitCode() == 2):
                invocationResult.setInvocationActualInstallLocation(invocationId, self._adapter.getInstallLocation())
            invocationResult.setInvocationLogfileLocation (invocationId, self._adapter.getInstallLog())
#            invocationResult.setInvocationTracefileLocation (invocationId,self._adapter.getInstallTraceFile())
        except:
            jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.SetResultsException"))
            self._killThread = 0

###############################################################################
#  wbi_cadapter - compareAndSetExitCode method
#  Compares the exit code from the adapter and sets exit code to IIP installer.
###############################################################################
    def compareAndSetExitCode ( self ):
        methodName="compareAndSetExitCode"
        jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.StartSetExitCode"))
        try:
            if self._adapter.getInstallExitCode() == 0:
                invocationResult.setInvocationExitCode(invocationId, invocationResult.SUCCESS)
                invocationController.getInstallationProgressListener().setInstallationProgressComplete()
            elif self._adapter.getInstallExitCode() == 1:
                invocationResult.setInvocationExitCode(invocationId, invocationResult.FAILED)
            elif self._adapter.getInstallExitCode() == 2:
                invocationResult.setInvocationExitCode(invocationId, invocationResult.PARTIAL_SUCCESS)
                invocationController.getInstallationProgressListener().setInstallationProgressComplete()
            elif self._adapter.getInstallExitCode() == 232 or self._adapter.getInstallExitCode() == 1000:
                invocationResult.setInvocationExitCode(invocationId, invocationResult.CANCELLED_BY_USER)
            else:
                invocationResult.setInvocationExitCode(invocationId, invocationResult.UNKNOWN)
            self._killThread = 0
        except:
            invocationResult.setInvocationExitCode(invocationId, invocationResult.UNKNOWN)
            jythonLogger.logMessage(className, methodName, jythonLogger.getLocaleString("IIPAdapterScript.SetExitCodeException"))
            self._killThread = 0

###############################################################################
#  wbi_cadapter - readProgressFromAdapter method
#  Call adapter's API to get current progress from the installer.
###############################################################################
    def readProgressFromAdapter ( self ):
        intCurrentProgress = self._adapter.getCurrentProgress()
        if self._progress < intCurrentProgress:
            self._progress = intCurrentProgress
        return self._progress
        
###############################################################################
#  wbi_cadapter - getKillThread method
#  Ends the progress reading thread.
###############################################################################
    def getKillThread ( self ):
        return self._killThread

##########################
#  Creates progress thread
##########################
def progressThread(delay, adapter):
    while adapter.getKillThread():
        time.sleep(delay)
        invocationController.getInstallationProgressListener().updateInstallationProgress(adapter.readProgressFromAdapter())        


##########################
#  Main
##########################
className="wbi_cadapter.py"
jythonLogger = invocationController.getJythonLogger()
productID="WBI"
primaryPakName="wbi.primary.pak"

# Get the additional Info
if ( invocationController.isGettingInfo() ):
  # Get Invocation Information
  jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.ObtainingAdditionalInfo"))
  conadapter = WBIContributionAdapter()
  primaryPakRelativePath = System.getProperty("file.separator") + productID + System.getProperty("file.separator") + primaryPakName
  BDoptions = invocationData.getBDOptionsForCurrentPlatformNoResolve (invocationId)
  # A java null is turned into a Python None value, throw exception if options is not resolved and null is received
  if BDoptions is None:
     raise exception
  if BDoptions != "":      
     jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.BDOptions", BDoptions))
     conadapter.setBDOptions(BDoptions)

  CMDoptions = invocationData.getCMDOptionsForCurrentPlatformNoResolve (invocationId)

  # A java null is turned into a Python None value, throw exception if options is not resolved and null is received
  if CMDoptions is None:
      raise exception
  if CMDoptions != "":
      conadapter.setCmdOptions(CMDoptions)
      jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.CMDOptions", CMDoptions))

  conadapter.setInvocationId (invocationId)
  conadapter.setIIPInvocationData(invocationData)
  invocationData.addInstallableContentForThisInvocation( conadapter.getInvocationInstallableContentFromImage( invocationId, contributionId, pathToPackage, primaryPakRelativePath, productID ) )
  invocationData.setOptionsWillBeOverriden( conadapter.getOptionsWillBeOverriden() )
  # Check if the override failure code:
  if conadapter.getReturnValue() == 1001:
      jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.OptionCannotOverride", CMDoptions))
      invocationResult.setInvocationMessage(invocationId, jythonLogger.getLocaleString("IIPAdapterScript.OptionCannotOverride", conadapter.getParamShouldNotBeOverriden()))
      invocationResult.setInvocationExitCode(invocationId, invocationResult.FAILED)
  jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.ObtainedAdditionalInfo"))
else:
  jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.Start"))
  contribution_adapter = wbi_cadapter()
  # Start the thread for returning progress
  thread.start_new_thread(progressThread,(5, contribution_adapter,))
  contribution_adapter.run()
  contribution_adapter.setResults()
  contribution_adapter.compareAndSetExitCode()
  jythonLogger.logMessage(className, __name__, jythonLogger.getLocaleString("IIPAdapterScript.End"))
